import 'package:flutter/material.dart';
import 'package:http/http.dart' as http; // http 패키지 임포트
import 'dart:convert'; // JSON 변환을 위해 임포트
import 'login_page.dart'; // 로그인 페이지 임포트 추가
import 'dart:convert' show utf8; // UTF-8 디코딩을 위해 임포트
import 'package:crypto/crypto.dart'; // crypto 패키지 임포트
import '../../config/config.dart'; // config 패키지 임포트
class SignUpPage extends StatefulWidget {
  const SignUpPage({Key? key}) : super(key: key);

  @override
  _SignUpPageState createState() => _SignUpPageState();
}

class _SignUpPageState extends State<SignUpPage> {
  // 체크박스 상태 (개인정보 수집 동의)
  bool _isAgreed = false; 
  // 입력값 
  String _username = '', _password = '', _confirmPassword = '', _nickname = '', _email = '';
  String _department = '', _introduceMyself = ''; 
  // 오류 메시지 (유효성 검사)
  String? _usernameError, _passwordError, _confirmPasswordError, _nicknameError, _emailError;

  // 유효성 검사 (아이디, 비밀번호, 이메일, 닉네임)
  bool _isUsernameValid(String username) =>
      RegExp(r'^(?![0-9])[A-Za-z0-9]{6,20}$').hasMatch(username);
  bool _isPasswordValidPattern(String password) =>
      RegExp(r"""^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d!@#$%^&*()_\-+=~`{}\[\]|\\:;\"'<>,.?/]{8,20}$""")
          .hasMatch(password);
  bool _isEmailValid(String email) =>
      RegExp(r'^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$').hasMatch(email);
  bool _isNicknameValid(String nickname) =>
      RegExp(r'^[A-Za-z가-힣0-9]{2,20}$').hasMatch(nickname);

  // label에 따른 입력값 검사
  void _validateInput(String label) {
    setState(() {
      if (label == 'Username') { // '아이디'
        _usernameError = _isUsernameValid(_username)
            ? null
            : 'Username must be 6–20 characters, letters and digits, and cannot start with a digit.';
      } else if (label == 'Password') { // '비밀번호'
        _passwordError = _isPasswordValidPattern(_password)
            ? null
            : 'Password must be 8–20 characters, including letters and digits.';
      } else if (label == 'Confirm Password') { // '비밀번호 확인'
        _confirmPasswordError = (_password == _confirmPassword)
            ? null
            : 'Passwords do not match.';
      } else if (label == 'Nickname') { // '닉네임'
        _nicknameError = _isNicknameValid(_nickname)
            ? null
            : '* Nickname must be 2–20 characters (letters, Korean, digits).';
      } else if (label == 'Email') { // '이메일'
        _emailError = _isEmailValid(_email)
            ? null
            : (_email.isNotEmpty ? 'Please enter a valid email address.' : null);
      }
    });
  }

  // 회원가입 요청 (Sign-up request)
  Future<void> _signUp() async {
    final url = 'https://eldsoft.com:8097/user/signup';

    // 체크박스 상태에 따라 mandatory_terms_yn 값 설정
    final mandatoryTermsYn = _isAgreed ? 'Y' : 'N';
    // 비밀번호 해싱 (SHA-256)
    final hashedPassword = _hashPassword(_password);

    final body = {
      "user_id": _username,
      "user_pw": hashedPassword,
      "nickname": _nickname,
      "user_email": _email,
      "department": _department,
      "introduce_myself": _introduceMyself,
      "mandatory_terms_yn": mandatoryTermsYn
    };

    try {
      final response = await http.post(
        Uri.parse(url),
        headers: {'Content-Type': 'application/json'},
        body: json.encode(body),
      );

      // 응답을 UTF-8로 디코딩
      final resBody = json.decode(utf8.decode(response.bodyBytes));

      if (response.statusCode == 200) {
        if (resBody['result'] == 'OK') {
          if (resBody['response_info']['msg_type'] == 'OK') {
            _showDialog('Sign-up Success', 'Your account has been created successfully.');
          } else {
            _showDialog('Sign-up Failed', '${resBody['response_info']['msg_content']}');
          }
        } else {
          _showDialog('Sign-up Failed', '${resBody['response_info']['msg_content']}');
        }
      } else {
        final errorData = json.decode(response.body);
        _showDialog('Sign-up Failed', errorData['message'] ?? 'Sign-up failed.');
      }
    } catch (error) {
      _showDialog('Network Error', 'Network error: $error');
    }
  }

  // 비밀번호 해싱 (SHA-256)
  String _hashPassword(String password) {
    final bytes = utf8.encode(password);
    final digest = sha256.convert(bytes);
    return digest.toString();
  }

  // 모달 다이얼로그
  void _showDialog(String title, String message) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          backgroundColor: Colors.white,
          title: Text(
            title,
            style: const TextStyle(
              fontSize: 20,
              fontWeight: FontWeight.bold,
              color: Colors.black,
            ),
          ),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              Text(
                message,
                style: const TextStyle(fontSize: 16, color: Colors.black),
              ),
              const SizedBox(height: 20),
              TextButton(
                onPressed: () {
                  Navigator.of(context).pop();
                  if (title == 'Sign-up Success') {
                    // 회원가입 성공이면 로그인 페이지로
                    Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => const LoginPage()));
                  }
                },
                style: TextButton.styleFrom(
                  foregroundColor: Colors.white,
                  backgroundColor: Colors.black,
                  padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                ),
                child: const Text('OK'),
              ),
            ],
          ),
        );
      },
    );
  }

  // 입력필드 공용 위젯 (label에 따라 유효성 검사)
  Widget _buildTextField(String label, Function(String) onChanged, {
    bool obscureText = false,
    String? errorText,
  }) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        TextField(
          onChanged: (value) {
            onChanged(value);
            _validateInput(label);
          },
          obscureText: obscureText,
          decoration: InputDecoration(
            labelText: label,
            labelStyle: const TextStyle(color: Colors.black),
            border: const OutlineInputBorder(),
            focusedBorder: const OutlineInputBorder(
              borderSide: BorderSide(color: Colors.black, width: 2.0),
            ),
            errorStyle: const TextStyle(color: Colors.red, fontSize: 12),
          ),
        ),
        if (errorText != null)
          Padding(
            padding: const EdgeInsets.only(top: 8.0),
            child: Text(errorText, style: const TextStyle(color: Colors.red, fontSize: 12)),
          ),
        const SizedBox(height: 16),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        title: const Text('ALL SCORE', style: TextStyle(color: Colors.white)),
        backgroundColor: Colors.black,
        leading: IconButton(
          icon: const Icon(Icons.chevron_left, color: Colors.white),
          onPressed: () => Navigator.pop(context),
        ),
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: SingleChildScrollView(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              const Text(
                'Sign Up', // '회원가입'
                style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold, color: Colors.black),
              ),
              const SizedBox(height: 32),

              _buildTextField('Username', (value) => _username = value, errorText: _usernameError),
              _buildTextField('Password', (value) => _password = value, obscureText: true, errorText: _passwordError),
              _buildTextField('Confirm Password', (value) => _confirmPassword = value, obscureText: true, errorText: _confirmPasswordError),
              _buildTextField('Nickname', (value) => _nickname = value, errorText: _nicknameError),
              _buildTextField('Email', (value) => _email = value, errorText: _emailError),
              _buildTextField('Affiliation (Optional)', (value) => _department = value),
              _buildTextField('Self-introduction (Optional)', (value) => _introduceMyself = value),
              const SizedBox(height: 16),

              // 개인정보 수집 및 이용 동의서
              const Text(
                'Consent to the Collection and Use of Personal Information',
                // '개인정보 수집 및 이용 동의서'
                style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: Colors.black),
              ),
              const SizedBox(height: 8),
              Container(
                height: 200,
                decoration: BoxDecoration(
                  border: Border.all(color: Colors.black.withOpacity(0.5)),
                  borderRadius: BorderRadius.circular(8),
                ),
                child: Scrollbar(
                  thickness: 5,
                  radius: const Radius.circular(5),
                  child: SingleChildScrollView(
                    child: const Padding(
                      padding: EdgeInsets.all(15.0),
                      child: Text(
                        Config.termsOfService,
                        textAlign: TextAlign.left,
                        style: const TextStyle(color: Colors.black),
                      ),
                    ),
                  ),
                ),
              ),
              const SizedBox(height: 16),
              // 동의 체크박스
              Row(
                children: [
                  Checkbox(
                    value: _isAgreed,
                    onChanged: (value) {
                      setState(() {
                        _isAgreed = value ?? false;
                      });
                    },
                  ),
                  const Text('I agree to the collection and use of my personal information.'),
                ],
              ),
              const SizedBox(height: 16),
              ElevatedButton(
                onPressed: _signUp, 
                style: ElevatedButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('Sign Up for ALLSCORE'),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
